@@ -84,6 +84,8 @@ static XtResource OFFW_resources[] = {
   /* key event  client ֤Ѥ callback */
   { XtNkeybackNotify, XtCCallback, XtRCallback, sizeof(caddr_t), 
     offset(keybackcallback), XtRCallback, (caddr_t)NULL },
+  { XtNtextNotify,	XtCCallback, XtRCallback, sizeof (XtCallbackList),
+	offset (m_lstTextCallback),	XtRCallback, (caddr_t) NULL },
   /* egg ߴ j-newline ݤ*/
   { XtNeggLikeNewline, XtCEggLikeNewline, XtRImmediate, sizeof( Boolean ),
     XtOffsetOf( OffthespotWinRec, offthespotWin.buffer.egg_like_newline ),
@@ -128,6 +130,8 @@ static XtResource OFFW_resources[] = {
     sizeof( Boolean ), 
     XtOffsetOf( OffthespotWinRec, offthespotWin.buffer.toggleControlMode ),
     XtRImmediate, ( XtPointer )FALSE },
+  {	XtNinput,	XtCInput,	XtRBoolean,	sizeof (Boolean),
+	offset (m_fInputFocus), XtRImmediate, (XtPointer) TRUE, },
 } ;
 
 #undef offset
@@ -189,6 +193,7 @@ static void OFFW_wmMessageCloseCallback
 ( Widget gw, caddr_t client, caddr_t caller ) ;
 static void OFFW_wmShellDestroyCallback
 ( Widget gw, caddr_t client, caddr_t caller ) ;
+static void		OffWin_fixcallback		(Widget, XtPointer, XtPointer) ;
 
 /* common style */
 extern void skkinput_bufferInit( struct skkinputBuffer *buffer ) ;
@@ -251,46 +256,46 @@ WidgetClass offthespotWinWidgetClass =
 
 static void OFFW_InitializeHistory( Widget gw )
 {
-  OffthespotWinWidget w = ( OffthespotWinWidget )gw ;
+	OffthespotWinWidget w = ( OffthespotWinWidget )gw ;
 
-  if( w->offthespotWin.historyAttribute == NULL ){
-    return ;
-  }
-  /* ꤷȤСġ*/
-  if( w->offthespotWin.historyAttribute->overthespot_like_input < 0 ){
-    /* X Resource ¦ꤹ롣*/
-    w->offthespotWin.buffer.overthespot_like_input = 
-      w->offthespotWin.overthespotLikeInput ;
-  } else {
-    /* ήѤ롣*/
-    w->offthespotWin.buffer.overthespot_like_input = 
-      w->offthespotWin.historyAttribute->overthespot_like_input ;
-  }
-  /* ҥȥν򤹤롣*/
-  w->offthespotWin.buffer.historybuffer =
-    w->offthespotWin.historyAttribute->history ;
-  w->offthespotWin.buffer.hist_start = 
-    w->offthespotWin.historyAttribute->history_start ;
-  w->offthespotWin.buffer.hist_end = 
-    w->offthespotWin.historyAttribute->history_end ;
-  w->offthespotWin.buffer.histcurbackbuffer =
-    w->offthespotWin.historyAttribute->histcurbak ;
-  w->offthespotWin.buffer.hist_cur = -1 ;
-  
-  /* ϥ⡼ɤν򤹤롣*/
-  w->offthespotWin.buffer.chat_adapter =
-    w->offthespotWin.historyAttribute->chat_adapter ;
-  w->offthespotWin.buffer.egg_like_newline =
-    w->offthespotWin.historyAttribute->egg_nl ;
-  
-  /* ̾⡼ɤν򤹤롣*/
-  w->offthespotWin.buffer.topbuffer->j_mode =
-    w->offthespotWin.historyAttribute->j_mode ;
-  w->offthespotWin.buffer.topbuffer->j_zenkaku =
-    w->offthespotWin.historyAttribute->j_zenkaku ;
-  w->offthespotWin.buffer.topbuffer->j_katakana_mode =
-    w->offthespotWin.historyAttribute->j_katakana_mode ;
-  return ;
+	if( w->offthespotWin.historyAttribute == NULL ){
+		return ;
+	}
+	/* ꤷȤСġ*/
+	if( w->offthespotWin.historyAttribute->overthespot_like_input < 0 ){
+		/* X Resource ¦ꤹ롣*/
+		w->offthespotWin.buffer.overthespot_like_input = 
+			w->offthespotWin.overthespotLikeInput ;
+	} else {
+		/* ήѤ롣*/
+		w->offthespotWin.buffer.overthespot_like_input = 
+			w->offthespotWin.historyAttribute->overthespot_like_input ;
+	}
+	/* ҥȥν򤹤롣*/
+	w->offthespotWin.buffer.historybuffer =
+		w->offthespotWin.historyAttribute->history ;
+	w->offthespotWin.buffer.hist_start = 
+		w->offthespotWin.historyAttribute->history_start ;
+	w->offthespotWin.buffer.hist_end = 
+		w->offthespotWin.historyAttribute->history_end ;
+	w->offthespotWin.buffer.histcurbackbuffer =
+		w->offthespotWin.historyAttribute->histcurbak ;
+	w->offthespotWin.buffer.hist_cur = -1 ;
+	
+	/* ϥ⡼ɤν򤹤롣*/
+	w->offthespotWin.buffer.chat_adapter =
+		w->offthespotWin.historyAttribute->chat_adapter ;
+	w->offthespotWin.buffer.egg_like_newline =
+		w->offthespotWin.historyAttribute->egg_nl ;
+	
+	/* ̾⡼ɤν򤹤롣*/
+	w->offthespotWin.buffer.topbuffer->j_mode =
+		w->offthespotWin.historyAttribute->j_mode ;
+	w->offthespotWin.buffer.topbuffer->j_zenkaku =
+		w->offthespotWin.historyAttribute->j_zenkaku ;
+	w->offthespotWin.buffer.topbuffer->j_katakana_mode =
+		w->offthespotWin.historyAttribute->j_katakana_mode ;
+	return ;
 }
 
 static void OFFW_Initialize 
@@ -495,27 +500,25 @@ static void OFFW_RealizeMinibuffer( Widget gw )
     font_height = w->offthespotWin.mfont_height ;
   }
   minibuffer_popup = XtVaCreatePopupShell
-    ( "minibuffer", wmcloseShellWidgetClass, gw, XtNinput, TRUE, 
-      XtNmappedWhenManaged, TRUE, 
-      XtNwidth, winwidth, XtNheight, winheight, 
-      XtNcolormap, gw->core.colormap, 
-      XtNreverseVideo, w->offthespotWin.reverse_video,
-      XtNbackground, w->offthespotWin.background,
-      XtNforeground, w->offthespotWin.foreground, NULL ) ;
-  XtAddCallback
-    ( minibuffer_popup, XtNwmcloseNotify, 
-      ( XtCallbackProc )OFFW_wmMessageCloseCallback, gw ) ;
-  XtAddCallback
-    ( minibuffer_popup, XtNdestroyNotify, 
-      ( XtCallbackProc )OFFW_wmShellDestroyCallback, gw ) ;
+    ( "minibuffer", wmcloseShellWidgetClass, gw,
+	  XtNinput,				w->offthespotWin.m_fInputFocus,
+      XtNmappedWhenManaged,	TRUE, 
+      XtNwidth,				winwidth,
+	  XtNheight,			winheight, 
+      XtNcolormap,			gw->core.colormap, 
+      XtNreverseVideo,		w->offthespotWin.reverse_video,
+      XtNbackground,		w->offthespotWin.background,
+      XtNforeground,		w->offthespotWin.foreground, NULL ) ;
+  XtAddCallback (minibuffer_popup, XtNwmcloseNotify, (XtCallbackProc) OFFW_wmMessageCloseCallback, gw) ;
+  XtAddCallback (minibuffer_popup, XtNdestroyNotify, (XtCallbackProc) OFFW_wmShellDestroyCallback, gw) ;
 
   /* βˤĤɽѤ widget 롣shellwidget ľ *
    * ܽ񤤤ƤϤޤäȤΤǡġ*/
   minibuffer_canvas = XtVaCreateManagedWidget
     ( "canvas", canvasWidgetClass, minibuffer_popup,
-      XtNreverseVideo, w->offthespotWin.reverse_video,
-      XtNbackground, w->offthespotWin.background,
-      XtNforeground, w->offthespotWin.foreground, NULL ) ;
+      XtNreverseVideo,		w->offthespotWin.reverse_video,
+      XtNbackground,		w->offthespotWin.background,
+      XtNforeground,		w->offthespotWin.foreground, NULL ) ;
   w->offthespotWin.minibuffer_probe = False ;
   /* 襤٥ȤåäƤ*/
   XtAddCallback
@@ -567,67 +570,72 @@ static void OFFW_RealizeMinibuffer( Widget gw )
 static void OFFW_Realize
 ( Widget gw, XtValueMask *valueMask, XSetWindowAttributes *attrs )
 {
-  OffthespotWinWidget w = ( OffthespotWinWidget )gw ;
-  CoreWidgetClass super =
-    ( CoreWidgetClass )XtClass( gw )->core_class.superclass ;
+	OffthespotWinWidget w = ( OffthespotWinWidget )gw ;
+	CoreWidgetClass super =
+		( CoreWidgetClass )XtClass( gw )->core_class.superclass ;
 
-  /* ŬڤʥɥѰդƤ*/
-  ( *super->core_class.realize )( gw, valueMask, attrs ) ;
+	/* ŬڤʥɥѰդƤ*/
+	( *super->core_class.realize )( gw, valueMask, attrs ) ;
 
-  /* Ѵ饤ȤꤵƤʤ⤷ϡեȤ꤬
-   * äƤˤϵưʤ*/
-  if( w->offthespotWin.client_window == None ){
-    XtDestroyWidget( gw ) ;
-    return ;
-  }
-  if( w->offthespotWin.focus_window == None ){
-    w->offthespotWin.focus_window = w->offthespotWin.client_window ;
-    /* եΥɥξƤʤΤʤ顢Ѱդ롣*/
-    if( !( w->offthespotWin.attribute_mask & CAClientArea ) )
-      XGetRectangleOfWindow
-	( XtDisplay( gw ), w->offthespotWin.focus_window, 
-	  &w->offthespotWin.client_area ) ;
-    OFFW_ReconfigureFocusPosition( gw ) ;
-  }
-  /* Ǥäȡgc 뤳ȤǤ롣*/
-  OFFW_CreateAllGCs( gw ) ;
+	/* Ѵ饤ȤꤵƤʤ⤷ϡեȤ꤬
+	 * äƤˤϵưʤ*/
+	if( w->offthespotWin.client_window == None ){
+		XtDestroyWidget( gw ) ;
+		return ;
+	}
 
-  /* Minibuffer 롣*/
-  OFFW_RealizeMinibuffer( gw ) ;
-  OFFW_CreateMinibufferGCs( gw ) ;
+	if (XtHasCallbacks (gw, XtNfixNotify) != XtCallbackHasSome) 
+		XtAddCallback (gw, XtNfixNotify, OffWin_fixcallback, NULL) ;
 
-  /* modeshell ɽΤΥåȤ롣*/
-  w->offthespotWin.modeshell_win = OFFW_CreateSubCanvas
-    ( gw, w->offthespotWin.client_window, 
-      &w->offthespotWin.modeshell_probe, 0 ) ;
-  w->offthespotWin.modeshell_width  = 1 ;
-  w->offthespotWin.modeshell_height = 1 ;
-  /* ʸѤΥХ3ɬפˤʤ롣ϥե
-     ˤϤĤȤˤʤ롣*/
-  w->offthespotWin.canvas_win = OFFW_CreateSubCanvas
-    ( gw, w->offthespotWin.client_window, 
-      &w->offthespotWin.canvas_probe, 0 ) ;
-  /* */
-  XMoveResizeWindow
-    ( XtDisplay( gw ), w->offthespotWin.canvas_win,
-      w->offthespotWin.client_area.x, w->offthespotWin.client_area.y,
-      w->offthespotWin.client_area.width,
-      w->offthespotWin.client_area.height ) ;
+	if( w->offthespotWin.focus_window == None ){
+		w->offthespotWin.focus_window = w->offthespotWin.client_window ;
+		/* եΥɥξƤʤΤʤ顢Ѱդ롣*/
+		if( !( w->offthespotWin.attribute_mask & CAClientArea ) )
+			XGetRectangleOfWindow
+				( XtDisplay( gw ), w->offthespotWin.focus_window, 
+				  &w->offthespotWin.client_area ) ;
+		OFFW_ReconfigureFocusPosition( gw ) ;
+	}
+	/* Ǥäȡgc 뤳ȤǤ롣*/
+	OFFW_CreateAllGCs( gw ) ;
+	
+	/* Minibuffer 롣*/
+	OFFW_RealizeMinibuffer( gw ) ;
+	OFFW_CreateMinibufferGCs( gw ) ;
+	
+	/* modeshell ɽΤΥåȤ롣*/
+	w->offthespotWin.modeshell_win = OFFW_CreateSubCanvas
+		( gw, w->offthespotWin.client_window, 
+		  &w->offthespotWin.modeshell_probe, 0 ) ;
+	w->offthespotWin.modeshell_width  = 1 ;
+	w->offthespotWin.modeshell_height = 1 ;
+	/* ʸѤΥХ3ɬפˤʤ롣ϥե
+	   ˤϤĤȤˤʤ롣*/
+	w->offthespotWin.canvas_win = OFFW_CreateSubCanvas
+		( gw, w->offthespotWin.client_window, 
+		  &w->offthespotWin.canvas_probe, 0 ) ;
+	XMoveResizeWindow
+		( XtDisplay( gw ), w->offthespotWin.canvas_win,
+		  w->offthespotWin.client_area.x, w->offthespotWin.client_area.y,
+		  w->offthespotWin.client_area.width,
+		  w->offthespotWin.client_area.height ) ;
      
-  /* եꤹΤʤ顢focus ѲѤ뤹٤Ƥ */
-  /* åȤrealizeǤʤФʤʤ*/
-  XtSetKeyboardFocus( w->offthespotWin.minibuffer_popup, gw ) ;
-  XtSetKeyboardFocus( w->offthespotWin.minibuffer_canvas, gw ) ;
-
-  /*
-   * ΰ֤ʤ⡼ɥƤʤ
-   */
-  OFFW_resetupModeWindow( gw ) ;
-  OFFW_ReconfigureColor( gw, CAColor | CAColormap ) ;
-
-  XMapWindow( XtDisplay( gw ), w->offthespotWin.canvas_win ) ;
-  w->offthespotWin.canvas_probe = True ;
-  return ;
+	/* եꤹΤʤ顢focus ѲѤ뤹٤Ƥ */
+	/* åȤrealizeǤʤФʤʤ*/
+	if (w->offthespotWin.m_fInputFocus) {
+		XtSetKeyboardFocus( w->offthespotWin.minibuffer_popup, gw ) ;
+		XtSetKeyboardFocus( w->offthespotWin.minibuffer_canvas, gw ) ;
+	}
+	
+	/*
+	 * ΰ֤ʤ⡼ɥƤʤ
+	 */
+	OFFW_resetupModeWindow (gw) ;
+	OFFW_ReconfigureColor (gw, CAForegroundPixel | CABackgroundPixel | CAColormap) ;
+	
+	XMapWindow (XtDisplay (gw), w->offthespotWin.canvas_win) ;
+	w->offthespotWin.canvas_probe = True ;
+	return ;
 }
 
 /*
@@ -822,46 +830,41 @@ static void OFFW_ResizeMinibufferAndModeshell( Widget gw )
  */
 static void OFFW_ReconfigureColor( Widget gw, unsigned long valuemask )
 {
-  OffthespotWinWidget w = ( OffthespotWinWidget )gw ;
-  unsigned long mask ;
-  XSetWindowAttributes xswa ;
+	OffthespotWinWidget w = ( OffthespotWinWidget )gw ;
+	unsigned long mask ;
+	XSetWindowAttributes xswa ;
 
-  mask = 0L ;
-  /* طʿѹ줿ΤǤ */
-  if( valuemask & CAColor ){
-    mask |= ( CWBackPixel | CWBorderPixel ) ;
-    xswa.background_pixel = w->offthespotWin.obackground ;
-    xswa.border_pixel     = w->offthespotWin.oforeground ;
-  }
-  /* 顼ޥåפѹ줿ΤǤ */
-  if( valuemask & CAColormap ){
-    mask |= CWColormap ;
-    xswa.colormap = w->offthespotWin.colormap ;
-  }
-  if( mask ){
-    /*   ֤󡣤ʤꤷʤȤʤΤġ      *
-     * աϳƥɥطʿȥ顼ޥåפκ* 
-     * Ƥ롣                                                    *
-     * ե꡼֥եȤ衢եȡ                          */
-    XChangeWindowAttributes
-      ( XtDisplay( gw ), w->offthespotWin.modeshell_win, 
-	mask, &xswa ) ;
-    XChangeWindowAttributes
-      ( XtDisplay( gw ), w->offthespotWin.canvas_win, 
-	mask, &xswa ) ;
-  }
-  /* ѹʤä顢GC Ϻľʤɤ 顼ޥ
-     פϤɤΤ */
-  if( !( valuemask & CAColor ) )
-    return ;
+	mask = 0L ;
+	if (valuemask & (CAForegroundPixel | CABackgroundPixel)) {
+		mask |= ( CWBackPixel | CWBorderPixel ) ;
+		xswa.background_pixel = w->offthespotWin.obackground ;
+		xswa.border_pixel     = w->offthespotWin.oforeground ;
+	}
+	/* 顼ޥåפѹ줿ΤǤ */
+	if( valuemask & CAColormap ){
+		mask |= CWColormap ;
+		xswa.colormap = w->offthespotWin.colormap ;
+	}
+	if( mask ){
+		XChangeWindowAttributes
+			( XtDisplay( gw ), w->offthespotWin.modeshell_win, 
+			  mask, &xswa ) ;
+		XChangeWindowAttributes
+			( XtDisplay( gw ), w->offthespotWin.canvas_win, 
+			  mask, &xswa ) ;
+	}
+	/* ѹʤä顢GC Ϻľʤɤ 顼ޥ
+	   פϤɤΤ */
+	if (! (valuemask & (CAForegroundPixel | CABackgroundPixel)))
+		return ;
 
-  if( XtIsRealized( gw ) ){
-    /* ޤȤäƤ GC Ʋ롣 minibuffer οä *
-     * ѹ٤ʡ  */
-    OFFW_ReleaseAllGCs( gw ) ;
-    OFFW_CreateAllGCs( gw ) ;
-  }
-  return ;
+	if (XtIsRealized (gw)){
+		/* ޤȤäƤ GC Ʋ롣 minibuffer οä *
+		 * ѹ٤ʡ  */
+		OFFW_ReleaseAllGCs( gw ) ;
+		OFFW_CreateAllGCs( gw ) ;
+	}
+	return ;
 }
 
 /*
@@ -1027,152 +1030,152 @@ static void OFFW_ChangeFocusWindow
 static int OFFW_ChangeAttributes
 ( Widget gw, unsigned long valuemask, struct ConvAttrs *values )
 {
-  OffthespotWinWidget w   = ( OffthespotWinWidget )gw ;
-  int ret = False, rec_col ;
+	OffthespotWinWidget w   = ( OffthespotWinWidget )gw ;
+	int ret = False, rec_col ;
 
-  if( valuemask & CAFocusWindow ){
+	if( valuemask & CAFocusWindow ){
 #ifdef DEBUG
-    fprintf
-      ( stderr, "Get focus window (%ld)\n", values->focus_window ) ;
+		fprintf ( stderr, "Get focus window (%ld)\n", values->focus_window ) ;
 #endif
-    /* եäƼºߤΡ Ȥ¤Ϥ⤦ä *
-     * ɬפȻפ*/
-    if( values->focus_window != None ){
-      /* ꤷȦΥեʪˤʤäƤ顢̣Ǥ */
-      /* ʤ ȤäǤ*/
-      if( w->offthespotWin.focus_window != None ){
-	OFFW_ChangeFocusWindow( gw, values->focus_window ) ;
-	w->offthespotWin.attribute_mask &= ~( CAStatusArea ) ;
-      } else {
-	w->offthespotWin.focus_window = values->focus_window ;
-      }
-      /* Ѵ饤ȤΰФ̵ä鼫Ѱդ *
-       * 줬ɤʤݾڤϤʤΤɡѴ *
-       * Ȥʤ̵ */
-      if( !(( w->offthespotWin.attribute_mask ) & CAClientArea )){
-	XGetRectangleOfWindow
-	  ( XtDisplay( gw ), w->offthespotWin.focus_window, 
-	    &w->offthespotWin.client_area ) ;
-      }
-      /* ơΰ褬ꤵƤʤä顢Ȥꤢϥ饤
-	 ΰѤ롣ΤޤޤǤФ˸ɤΤ
-	 ץꥱ¦Ȥ뤳ȡ*/
-      if( !(( w->offthespotWin.attribute_mask ) & CAStatusArea ) ){
-	w->offthespotWin.status_area = w->offthespotWin.client_area ; 
-      }
-      OFFW_ReconfigureFocusPosition( gw ) ;
-      ret = True ;
-    }
-  }
-  if( valuemask & CAStatusArea ){
-    /* ɸνä⤷ɽΰ褬Ƥ硣*/
-    if( values->status_area.x != w->offthespotWin.status_area.x ||
-        values->status_area.y != w->offthespotWin.status_area.y ||
-        values->status_area.width  != w->offthespotWin.status_area.width || 
-        values->status_area.height != w->offthespotWin.status_area.height ){
-      w->offthespotWin.status_area.x = values->status_area.x ;
-      w->offthespotWin.status_area.y = values->status_area.y ;
-      w->offthespotWin.status_area.width  = values->status_area.width ;
-      w->offthespotWin.status_area.height = values->status_area.height ;
-      w->offthespotWin.attribute_mask |= CAStatusArea ;
-      ret = True ;
-    }
-  }
-  rec_col = False ;
-  /* 顼ޥåפѹ׵᤬褿硣*/
-  if( ( valuemask & CAColormap ) &&
-     w->offthespotWin.colormap != values->colormap ){
-    w->offthespotWin.colormap = values->colormap ;
-    rec_col = True ;
-  }
-  /* طʿʿѹ׵᤬褿硣*/
-  if( ( valuemask & CAColor ) &&
-     ( w->offthespotWin.obackground != values->background ||
-       w->offthespotWin.oforeground != values->foreground ) ){
-    w->offthespotWin.obackground = values->background ;
-    w->offthespotWin.oforeground = values->foreground ;
-    rec_col = True ;
-  }
-  if( XtIsRealized( gw ) && rec_col ){
-    OFFW_ReconfigureColor( gw, valuemask ) ;
-    ret = True ;
-  }
-  /* Ѵ饤ȤɽΰνΤ褿ν*/
-  if( valuemask & CAClientArea ){
-    /* ɸνä⤷ɽΰ褬Ƥ硣*/
-    if( values->client_area.x != w->offthespotWin.client_area.x ||
-        values->client_area.y != w->offthespotWin.client_area.y ||
-        values->client_area.width != w->offthespotWin.client_area.width || 
-        values->client_area.height != w->offthespotWin.client_area.height ){
-      /* Ѵ饤ȤɽΰѹФƤ*/
-      w->offthespotWin.client_area    = values->client_area ;
+		/* եäƼºߤΡ Ȥ¤Ϥ⤦ä *
+		 * ɬפȻפ*/
+		if( values->focus_window != None ){
+			/* ꤷȦΥեʪˤʤäƤ顢̣Ǥ */
+			/* ʤ ȤäǤ*/
+			if( w->offthespotWin.focus_window != None ){
+				OFFW_ChangeFocusWindow( gw, values->focus_window ) ;
+				w->offthespotWin.attribute_mask &= ~( CAStatusArea ) ;
+			} else {
+				w->offthespotWin.focus_window = values->focus_window ;
+			}
+			/* Ѵ饤ȤΰФ̵ä鼫Ѱդ *
+			 * */
+			if( !(( w->offthespotWin.attribute_mask ) & CAClientArea )){
+				XGetRectangleOfWindow
+					( XtDisplay( gw ), w->offthespotWin.focus_window, 
+					  &w->offthespotWin.client_area ) ;
+			}
+			/* ơΰ褬ꤵƤʤä顢Ȥꤢϥ饤
+			   ΰѤ롣ΤޤޤǤФ˸ɤΤ
+			   ץꥱ¦Ȥ뤳ȡ*/
+			if( !(( w->offthespotWin.attribute_mask ) & CAStatusArea ) ){
+				w->offthespotWin.status_area = w->offthespotWin.client_area ; 
+			}
+			OFFW_ReconfigureFocusPosition( gw ) ;
+			ret = True ;
+		}
+	}
+	if( valuemask & CAStatusArea ){
+		/* ɸνä⤷ɽΰ褬Ƥ硣*/
+		if( values->status_area.x != w->offthespotWin.status_area.x ||
+			values->status_area.y != w->offthespotWin.status_area.y ||
+			values->status_area.width  != w->offthespotWin.status_area.width || 
+			values->status_area.height != w->offthespotWin.status_area.height ){
+			w->offthespotWin.status_area.x = values->status_area.x ;
+			w->offthespotWin.status_area.y = values->status_area.y ;
+			w->offthespotWin.status_area.width  = values->status_area.width ;
+			w->offthespotWin.status_area.height = values->status_area.height ;
+			w->offthespotWin.attribute_mask |= CAStatusArea ;
+			ret = True ;
+		}
+	}
+	rec_col = False ;
+	/* 顼ޥåפѹ׵᤬褿硣*/
+	if( ( valuemask & CAColormap ) &&
+		w->offthespotWin.colormap != values->colormap ){
+		w->offthespotWin.colormap = values->colormap ;
+		rec_col = True ;
+	}
+	/* طʿʿѹ׵᤬褿硣*/
+	if ((valuemask & CAForegroundPixel) &&
+		w->offthespotWin.oforeground != values->foreground){
+		w->offthespotWin.oforeground = values->foreground ;
+		rec_col = True ;
+	}
+	if ((valuemask & CABackgroundPixel) &&
+		w->offthespotWin.obackground != values->background) {
+		w->offthespotWin.obackground = values->background ;
+		rec_col = True ;
+	}
+	if( XtIsRealized( gw ) && rec_col ){
+		OFFW_ReconfigureColor( gw, valuemask ) ;
+		ret = True ;
+	}
+	/* Ѵ饤ȤɽΰνΤ褿ν*/
+	if( valuemask & CAClientArea ){
+		/* ɸνä⤷ɽΰ褬Ƥ硣*/
+		if( values->client_area.x != w->offthespotWin.client_area.x ||
+			values->client_area.y != w->offthespotWin.client_area.y ||
+			values->client_area.width != w->offthespotWin.client_area.width || 
+			values->client_area.height != w->offthespotWin.client_area.height ){
+			/* Ѵ饤ȤɽΰѹФƤ*/
+			w->offthespotWin.client_area    = values->client_area ;
 #ifdef DEBUG
-      fprintf( stderr, "Get client area pos:(%d,%d), size:%d x %d.\n",
-	       w->offthespotWin.client_area.x,
-	       w->offthespotWin.client_area.y,
-	       w->offthespotWin.client_area.width,
-	       w->offthespotWin.client_area.height ) ;
+			fprintf( stderr, "Get client area pos:(%d,%d), size:%d x %d.\n",
+					 w->offthespotWin.client_area.x,
+					 w->offthespotWin.client_area.y,
+					 w->offthespotWin.client_area.width,
+					 w->offthespotWin.client_area.height ) ;
 #endif
-      w->offthespotWin.attribute_mask |= CAClientArea ;
-      ret = True ;
-    }
-  }
-  /* ɸȤ׵᤬ƤνԤ*/
-  if( valuemask & CASpotLocation ){
-    if( w->offthespotWin.focus_window != None ){
-      int prev_focus ;
+			w->offthespotWin.attribute_mask |= CAClientArea ;
+			ret = True ;
+		}
+	}
+	/* ɸȤ׵᤬ƤνԤ*/
+	if( valuemask & CASpotLocation ){
+		if( w->offthespotWin.focus_window != None ){
+			int prev_focus ;
 #if defined(DEBUG)
-      fprintf( stderr, "CASPOT nanode FOCUSWINDOW miru (%ld).\n",
-	      w->offthespotWin.focus_window ) ;
+			fprintf( stderr, "CASPOT nanode FOCUSWINDOW miru (%ld).\n",
+					 w->offthespotWin.focus_window ) ;
 #endif
-      prev_focus = w->offthespotWin.is_focus ;
-      w->offthespotWin.is_focus = focusCheck( gw ) ;
-      if( prev_focus != w->offthespotWin.is_focus ){
-	ret = True ;
-      }
-    }
-    OFFW_ReconfigureFocusPosition( gw ) ;
-    if( values->spot_x != w->offthespotWin.spot_x ||
-	values->spot_y != w->offthespotWin.spot_y ){
+			prev_focus = w->offthespotWin.is_focus ;
+			w->offthespotWin.is_focus = focusCheck( gw ) ;
+			if( prev_focus != w->offthespotWin.is_focus ){
+				ret = True ;
+			}
+		}
+		OFFW_ReconfigureFocusPosition( gw ) ;
+		if( values->spot_x != w->offthespotWin.spot_x ||
+			values->spot_y != w->offthespotWin.spot_y ){
 #ifdef DEBUG
-      fprintf( stderr, "Get Basho Shitei meirei.\n" ) ;
+			fprintf( stderr, "Get Basho Shitei meirei.\n" ) ;
 #endif
-      /* ֤뤵*/
-      w->offthespotWin.spot_x = values->spot_x ;
-      w->offthespotWin.spot_y = values->spot_y ;
-      ret = True ;
-    }
-  }
-  /* եȾνäν*/
-  if( valuemask & CAFonts ){
+			w->offthespotWin.spot_x = values->spot_x ;
+			w->offthespotWin.spot_y = values->spot_y ;
+			ret = True ;
+		}
+	}
+	/* եȾνäν*/
+	if( valuemask & CAFonts ){
 #ifdef DEBUG
-    fprintf( stderr, "FONT WO KIRIKAERU.\n" ) ;
+		fprintf( stderr, "FONT WO KIRIKAERU.\n" ) ;
 #endif
-    /* եȾ򹹿롣λ饤¦ΥեȤξ
-     * ƹ碌ΤȤ롣㤨饤ȤեȤ
-     * ꤷƤʤƥǥեȤΥեȤȤȤˤʤꡢΥե
-     * Ȥι⤵ʤƤġ(ɤΤ) */
-    if( fontMgr_GetFontSetInfo
-	( values->fontset,
-	  &w->offthespotWin.font_height,
-	  &w->offthespotWin.font_ascent ) ){
-      /* եȥåȤ򥳥ԡ롣*/
-      fontMgr_CopyFontSet
-	( XtDisplay( gw ), w->offthespotWin.fontset, values->fontset ) ;
-      /* ߥ˥Хåեȥ⡼ɥ礭롣*/
-      if( XtIsRealized( gw ) )
-	OFFW_ResizeMinibufferAndModeshell( gw ) ;
-      ret = True ;
-    }
-  }
-  /* եȤι⤵˴ؤƤˤϡġ*/
-  if( valuemask & CALineSpacing ){
-    if( values->linespacing > 0 ){
-      w->offthespotWin.font_height = values->linespacing ;
-      ret = True ;
-    }
-  }
-  return ret ;
+		/* եȾ򹹿롣λ饤¦ΥեȤξ
+		 * ƹ碌ΤȤ롣㤨饤ȤեȤ
+		 * ꤷƤʤƥǥեȤΥեȤȤȤˤʤꡢΥե
+		 * Ȥι⤵ʤƤġ(ɤΤ) */
+		if( fontMgr_GetFontSetInfo
+			( values->fontset,
+			  &w->offthespotWin.font_height,
+			  &w->offthespotWin.font_ascent ) ){
+			/* եȥåȤ򥳥ԡ롣*/
+			fontMgr_CopyFontSet
+				( XtDisplay( gw ), w->offthespotWin.fontset, values->fontset ) ;
+			/* ߥ˥Хåեȥ⡼ɥ礭롣*/
+			if( XtIsRealized( gw ) )
+				OFFW_ResizeMinibufferAndModeshell( gw ) ;
+			ret = True ;
+		}
+	}
+	/* եȤι⤵˴ؤƤˤϡġ*/
+	if( valuemask & CALineSpacing ){
+		if( values->linespacing > 0 ){
+			w->offthespotWin.font_height = values->linespacing ;
+			ret = True ;
+		}
+	}
+	return ret ;
 }
 
 /*
@@ -1766,3 +1769,14 @@ static void OFFW_wmShellDestroyCallback
   XtRemoveAllCallbacks( gw, XtNdestroyNotify ) ;
   return ;
 }
+
+static void
+OffWin_fixcallback (
+	register Widget		gw,
+	register XtPointer	client,
+	register XtPointer	caller)
+{
+	XtCallCallbacks (gw, XtNtextNotify, caller) ;
+	return ;
+}
+
